#!/usr/bin/python -u
# vim: fileencoding=utf-8:nomodified:nowrap:textwidth=0:foldmethod=marker:foldcolumn=4:ruler:showcmd:lcs=tab\:|- list:tabstop=8:noexpandtab:nosmarttab:softtabstop=0:shiftwidth=0
import sys

def calculate_checksum(data):
	return (-(sum(data) & 0xFF)) & 0xFF

def main():
	if not len(sys.argv) in (2,3,4,5):
		progname = sys.argv[0]
		print(f"Usage: {progname} <binary_file> [<offset_hex> [<lenght_hex> [<line_lenght_hex>]]")
		print(f"""Example:
	{progname} test.bin 
		prints whole test.bin as IHEX starting at 0000
	{progname} test.bin C00 A >test.hex
		prints 10 (0xA) bytes from test.bin as IHEX starting at 0C00 hexa to file test.hex
		""")
		sys.exit(1)

	binary_file = sys.argv[1]
	offset = 0
	length = 0
	data_length = 0
	if len(sys.argv) >=3:
		offset = int(sys.argv[2], 16)
	if len(sys.argv) >=4:
		length = int(sys.argv[3], 16)
	if len(sys.argv) >=5:
		data_length = int(sys.argv[4], 16)

	with open(binary_file, "rb") as f:
		data = f.read()
	if not length:
		length=len(data)

	address = offset
	if not data_length:
		data_length = 16

	while data and (length):
		if data_length > length:
			data_length = length
		length -= data_length
		
		chunk = data[:data_length]
		data = data[data_length:]
		
		record = [len(chunk), address >> 8, address & 0xFF, 0x00] + list(chunk)
		checksum = calculate_checksum(record)
		
		record.append(checksum)
		
		hex_record = ":" + "".join("{:02X}".format(byte) for byte in record)
		print(hex_record)
		
		address += len(chunk)

	# End of File record
	print(":00000001FF")

if __name__ == "__main__":
	main()

